package org.netkernel.mod.architecture;

import java.util.concurrent.atomic.AtomicInteger;

import org.netkernel.ext.system.representation.IRepDeployedModules;
import org.netkernel.layer0.meta.IEndpointMeta;
import org.netkernel.layer0.meta.IModuleMeta;
import org.netkernel.layer0.meta.IPhysicalEndpointMeta;
import org.netkernel.layer0.meta.IPrototypeMeta;
import org.netkernel.layer0.meta.IRepresentationMeta;
import org.netkernel.layer0.meta.ISpaceElements;
import org.netkernel.layer0.meta.ISpaceMeta;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.IHDSNodeList;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.urii.IEndpoint;
import org.netkernel.urii.IIdentifier;
import org.netkernel.urii.IMetaRepresentation;
import org.netkernel.urii.ISpace;

public class Utils
{
	public static void generateContextMenu(HDSBuilder b, INKFRequestContext aContext, String aMenuId, String aEntityType) throws Exception
	{
		b.pushNode("div");
		b.addNode("@style", "display: none;");
		b.addNode("@class", "contextMenu");
		b.addNode("@id", aMenuId);
		b.pushNode("ul");
		
		ISERep config = aContext.source("active:seConfig",ISERep.class);
		IHDSNodeList rootViews=config.getViewsFor(aEntityType);
		for (IHDSNode view: rootViews)
		{
			if (view.getFirstNode("separator")!=null)
			{	b.addNode("hr", null);
			}
			else
			{
				String name=view.getFirstValue("name").toString();
				String id=view.getFirstValue("id").toString();
				b.pushNode("li",name);
				b.addNode("@id",id);
				b.popNode();
			}
		}
		
		b.popNode();
		b.popNode();
	}
	
	private static AtomicInteger sUniqueId=new AtomicInteger();
	public static String getUniqueId()
	{	return Integer.toString(sUniqueId.getAndIncrement());
	}
	
	public static String iconClassForEntity(IMetaRepresentation meta)
	{	String icon="";
		if (meta instanceof IRepresentationMeta)
		{	icon="fa fa-picture-o";
		}
		else if (meta instanceof IPrototypeMeta)
		{	icon="fnk fnk-prototype";
		}
		else if (meta instanceof IEndpointMeta)
		{	icon="fnk fnk-endpoint";
		}
		else if (meta instanceof ISpaceMeta)
		{	icon="fa fa-cloud";
		}
		else if (meta instanceof IModuleMeta)
		{	icon="fnk fnk-cube";
			//icon="ui-icon-ae ui-icon-ae-module";
		}
		else if (meta instanceof IPhysicalEndpointMeta)
		{	icon="fnk fnk-component";
		}
		return icon;
	}
	
	public static String iconClassForType(String t)
	{	String icon="";
		if (t.equals("representation"))
		{	icon="fa fa-picture-o";
		}
		else if (t.equals("prototype"))
		{	icon="fnk fnk-prototype";
		}
		else if (t.equals("representation"))
		{	icon="fa fa-picture-o";
		}
		else if (t.equals("space"))
		{	icon="fa fa-cloud";
		}
		else if (t.equals("module"))
		{	icon="fnk fnk-cube"; //"ui-icon-ae ui-icon-ae-module";
		}
		else if (t.equals("physical"))
		{	icon="fnk fnk-component";
		}
		else if (t.equals("trace"))
		{	icon="ui-icon-ae ui-icon-ae-trace";
		}
		else if (t.equals("request"))
		{	icon="ui-icon-ae ui-icon-ae-request";
		}
		else if (t.equals("breakpoint"))
		{	icon="ui-icon-ae ui-icon-ae-breakpoint";
		}
		return icon;
	}

	public static String menuClassForEntity(IMetaRepresentation meta)
	{
		String entityClass=null;
		if (meta instanceof IRepresentationMeta)
		{	entityClass="el-rep";
		}
		else if (meta instanceof IPrototypeMeta)
		{	entityClass="el-proto";
		}
		else if (meta instanceof IEndpointMeta)
		{	entityClass="el-ep";
		}
		else if (meta instanceof ISpaceMeta)
		{	entityClass="el-space";
		}
		else if (meta instanceof IModuleMeta)
		{	entityClass="el-module";
		}
		else if (meta instanceof IPhysicalEndpointMeta)
		{	entityClass="el-physical";
		}
		return entityClass;
	}
	
	public static String menuClassForType(String t)
	{
		String entityClass=null;
		if (t.equals("representation"))
		{	entityClass="el-rep";
		}
		else if (t.equals("prototype"))
		{	entityClass="el-proto";
		}
		else if (t.equals("endpoint"))
		{	entityClass="el-ep";
		}
		else if (t.equals("space"))
		{	entityClass="el-space";
		}
		else if (t.equals("module"))
		{	entityClass="el-module";
		}
		else if (t.equals("physical"))
		{	entityClass="el-physical";
		}
		else if (t.equals("trace"))
		{	entityClass="el-trace";
		}
		else if (t.equals("request"))
		{	entityClass="el-request";
		}
		else if (t.equals("breakpoint"))
		{	entityClass="el-breakpoint";
		}
		return entityClass;
	}
	
	
	
	public static String findOriginatingPhysicalEndpoint(String aElementHash, IRepDeployedModules aModules)
	{
		int j=aElementHash.lastIndexOf('/');
		String elementId=aElementHash.substring(j+1);
		String spaceHash=aElementHash.substring(0,j);
		String resultSpaceHash=spaceHash;
		String resultPhysicalHash=null;
		
		do
		{	ISpaceMeta spaceMeta=(ISpaceMeta)aModules.metadataForHash(spaceHash);
			spaceHash=null;
			ISpaceElements spaceElements=spaceMeta.getElements();
			for (int i=0; i<spaceElements.size(); i++)
			{	IIdentifier eid=spaceElements.getIdentifier(i);
				if (eid!=null && eid.toString().equals(elementId))
				{	IEndpoint physicalEp=spaceElements.getPhysicalEndpoint(i);
					resultPhysicalHash=aModules.hashForComponent(physicalEp);
					ISpace delegatedSpace=spaceElements.getDelegatedSpace(i);
					if (delegatedSpace!=null)
					{	resultSpaceHash=aModules.hashForComponent(delegatedSpace);
						spaceHash=resultSpaceHash;
					}
					else
					{	break;
					}
				}
			}
		} while(spaceHash!=null);
		return resultPhysicalHash;
	}
	
}
