package org.netkernel.mod.architecture.bits.staticstructure;

import java.awt.Dimension;
import java.awt.geom.Dimension2D;
import java.util.*;

import org.netkernel.ext.system.representation.IRepDeployedModules;
import org.netkernel.ext.system.representation.impl.SpaceGraph;
import org.netkernel.layer0.meta.IModuleMeta;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.impl.HDSBuilder;

public class DefaultDecorator implements IStaticStructureDiagramDecorator
{
	protected IHDSNode mParams;
	protected IRepDeployedModules mModules;
	protected SpaceGraph mSpaceGraph;
	protected Set<String> mRootModuleHashes=new HashSet<String>();
	protected Set<String> mLibraryModules = new HashSet<String>();
	
	public void initialise(IHDSNode aParams, IRepDeployedModules aModules, SpaceGraph aSpaceGraph, IHDSNode aSettings, INKFRequestContext aContext) throws Exception
	{	mParams=aParams;
		mModules=aModules;
		mSpaceGraph=aSpaceGraph;
		
		mRootModuleHashes.clear();
		for (Object o : mParams.getValues("root"))
		{	String h=(String)o;
			mRootModuleHashes.add(getModuleHash(h));		
		}
		
		mLibraryModules.clear();
		for (IHDSNode node: aSettings.getNodes("/explorer/libraries/library"))
		{	String library=(String)node.getValue();
			mLibraryModules.add(library);
		}
	}
	
	private static String getModuleHash(String aHash)
	{	int i=aHash.indexOf('/');
		return i>0?aHash.substring(0,i):aHash;
	}

	public boolean isLibrarySpace(String aSpaceHash)
	{
		boolean system=false;
		int i=aSpaceHash.indexOf('/');
		String moduleHash=aSpaceHash.substring(0,i);
		IModuleMeta moduleMeta=(IModuleMeta)mModules.metadataForHash(moduleHash);
		String id=moduleMeta.getIdentifier();
		system=mLibraryModules.contains(id);

		if (system)
		{	if (mRootModuleHashes.contains(getModuleHash(aSpaceHash)))
			{	system=false;
			}
		}
		else
		{	String depth=(String)mParams.getFirstValue("depth");
			if (depth.equals("module") && !mRootModuleHashes.contains(getModuleHash(aSpaceHash)))
			{	system=true;
			}
		}	
		return system;
	}

	public boolean mergeLibraries()
	{	boolean result=false;
		String lib=(String)mParams.getFirstValue("lib");
		if (lib!=null &&  lib.equals("merge"))
		{	result=true;
		}
		return result;
	}
	
	public boolean showRoot(String aSpaceHash)
	{	return true;
	}

	public boolean showEdge(String aFromSpaceHash, String aToSpaceHash, int aDepth)
	{
		boolean result=true;
		if (result)
		{	result=!isLibrarySpace(aFromSpaceHash);
		}
		if (result && !showLibraries())
		{	result=!isLibrarySpace(aToSpaceHash);
		}
		
		String depth=(String)mParams.getFirstValue("depth");
		if (depth.equals("module"))
		{	String toModHash=getModuleHash(aToSpaceHash);
			if (mRootModuleHashes.contains(toModHash))
			{	result=true;
			}
			else if (result && showLibraries())
			{	String fromModHash=getModuleHash(aFromSpaceHash);
				result=mRootModuleHashes.contains(fromModHash);
			}
			else
			{	result=false;
			}
		}
		else if (depth.equals("all"))
		{
		}
		else
		{	if (result)
			{	int di=Integer.parseInt(depth);
				result=aDepth<di;
			}
		}
		
		return result;
	}

	public boolean showLibraries()
	{	boolean result=false;
		String lib=(String)mParams.getFirstValue("lib");
		if (lib!=null && (lib.equals("show") || lib.equals("merge")))
		{	result=true;
		}
		return result;
	}

	public boolean showModules()
	{	boolean result=false;
		String module=(String)mParams.getFirstValue("module");
		if (module!=null && module.equals("show") )
		{	result=true;
		}
		return result;
	}
	
	public void decorateEdge(HDSBuilder aBuilder, GraphEdge aEdge, String aId)
	{	
	}
	
	public void decorateSpace(HDSBuilder aBuilder, GraphNode aSpace, String aId)
	{
	}
	
	public void decorateSVG(HDSBuilder aBuilder, String aId)
	{
	}



	public boolean shade()
	{	String s=(String)mParams.getFirstValue("shading");
		return s==null || !s.equals("off");
	}

}
