<?xml version="1.0" encoding="UTF-8" ?>

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xrl="http://netkernel.org/xrl">
	<xsl:output method="xml" />
	<xsl:param name="validation"/>

	<xsl:template match="/*" >
		<div class="book-text">
			<style type="text/css"><xsl:comment>
				.error, .error td { background: red; }
				.etext { font-weight: bold; }
				.data-table { background: #ccc; padding: 2px; }
				.data-table tr { background: none; }
				.book-text a:link, .book-text a:visited,  .book-text a:hover { text-decoration: underline; } 
			</xsl:comment></style>
			<form method="post">
				<input type="hidden" name="page" value="{page}"/>

				<xsl:choose>
					<xsl:when test="page='1'">
						<xsl:call-template name="page1"/>
					</xsl:when>
					<xsl:when test="page='2'">
						<xsl:call-template name="page2"/>
					</xsl:when>
					<xsl:when test="page='3'">
						<xsl:call-template name="page3"/>
					</xsl:when>
					<xsl:when test="page='4'">
						<xsl:call-template name="page4"/>
					</xsl:when>
				</xsl:choose>
				
			</form>
		</div>
	</xsl:template>
	
	<xsl:template name="page1">
		<xrl:include>
		 <xrl:identifier>active:controlPanelInfoBoxOpen</xrl:identifier>
		 <xrl:argument name="content">
		   <div>
		   <p>Welcome to the new module wizard. This wizard will guide you through the process of creating and deploying a new module.
		   Don't worry about making mistakes, this wizard will validate each step.</p>
		   <p>This first page gathers the information to populate the &lt;meta&gt; section at the top of the module.xml.</p>
		   </div>
		 </xrl:argument>
		</xrl:include>
		<table class="table">
		      <tr>
				<xsl:if test="$validation/v/urn"><xsl:attribute name="class">error</xsl:attribute></xsl:if>
		        <td class="label">Identifier:</td>
		        <td>
		          <input type="text" name="urn" size="40" value="{urn}"/>
		        </td>
		        <td class="comment">
		        	<xsl:if test="$validation/v/urn"><div class="etext"><xsl:value-of select="$validation/v/urn"/></div></xsl:if>
		          The globally unique identifier for the module.<br/>
		
		          A good standard would follow Java package name convention or DNS convention
		          such as urn:org:myorg:myproject:mymodule
		        </td>
		      </tr>
		      <tr>
		      	<xsl:if test="$validation/v/version"><xsl:attribute name="class">error</xsl:attribute></xsl:if>
		        <td class="label">Version:</td>
		        <td>
		          <input type="text" name="version" size="10" value="{version}"/>
		        </td>
		        <td class="comment">
		        	<xsl:if test="$validation/v/version"><div class="etext"><xsl:value-of select="$validation/v/version"/></div></xsl:if>
		          Initial module version number.<br/>
		          Should be in the form a.b.c.d where a, b, c, d are integers.  
		        </td>
		      </tr>
			  <tr>
			  	<xsl:if test="$validation/v/name"><xsl:attribute name="class">error</xsl:attribute></xsl:if>
		        <td class="label">Name:</td>
		        <td>
		          <input type="text" name="name" size="40" value="{name}"/>
		        </td>
		        <td class="comment">
		        <xsl:if test="$validation/v/name"><div class="etext"><xsl:value-of select="$validation/v/name"/></div></xsl:if>
		        A short name used to identify the module in the system.</td>
		      </tr>
		      <tr>
		        <td class="label">Description:</td>
		        <td>
		          <textarea name="desc" cols="40" rows="2" style="width: 100%">
		          	<xsl:choose>
		          		<xsl:when test="string-length(desc)>0"><xsl:value-of select="desc"/>
		          		</xsl:when>
		          		<xsl:otherwise><xsl:text> </xsl:text></xsl:otherwise>
		          	</xsl:choose>
		          </textarea>
		        </td>
		        <td class="comment">A long description for the module.<br/> Use this to describe the purpose and contents of the module.</td>
		      </tr>
		      <tr>
		        <td colspan="3">
		          <button class="btn btn-primary" type="submit" value="next &gt;">next <span class="fa fa-arrow-right"> </span></button>
		        </td>
		      </tr>			
		</table>
	</xsl:template>
	
	<xsl:template name="page2">
		<xrl:include>
		 <xrl:identifier>active:controlPanelInfoBoxOpen</xrl:identifier>
		 <xrl:argument name="content">
		   <div>
		   <p>Now we need to collect information on the paths used by the new module.</p>
		   </div>
		 </xrl:argument>
		</xrl:include>
		
		<table class="table">
		      <tr>
		      	<xsl:if test="$validation/v/source"><xsl:attribute name="class">error</xsl:attribute></xsl:if>
		        <td class="label">Source&#160;Directory:</td>
		        <td>
		          <input type="text" name="source" size="50" value="{source}"/>
		        </td>
		        <td class="comment">
		       	 <xsl:if test="$validation/v/source"><div class="etext"><xsl:value-of select="$validation/v/source"/></div></xsl:if>
		          The physical directory where the module will be created in the file system.
		          <br/>This directory will be created and so must not already exist.
		        </td>
		      </tr>
		
			  <tr>
			  	<xsl:if test="$validation/v/name"><xsl:attribute name="class">error</xsl:attribute></xsl:if>
		        <td class="label">Fulcrum:</td>
		        <td>
		          <select name="fulcrum" size="1">
		          	<option value="">none</option>
		          	<option value="HTTPFulcrum">
		          		<xsl:if test="fulcrum='HTTPFulcrum'"><xsl:attribute name="selected"/></xsl:if>
		          		FrontEnd Fulcrum</option>
		          	<option value="HTTPAdminFulcrum">
		          		<xsl:if test="fulcrum='HTTPAdminFulcrum'"><xsl:attribute name="selected"/></xsl:if>
		          		BackEnd Admin Fulcrum</option>
		          </select>
		        </td>
		        <td class="comment">
		        If you wish to have this module imported by a fulcrum, select the target fulcrum from this list. </td>
		      </tr>
		      <tr>
				<xsl:if test="$validation/v/external"><xsl:attribute name="class">error</xsl:attribute></xsl:if>
		        <td class="label">External&#160;Path:</td>
		        <td>
		          <input type="text" name="external" size="50" value="{external}"/>
		        </td>
		        <td class="comment">
		        	<xsl:if test="$validation/v/external"><div class="etext"><xsl:value-of select="$validation/v/external"/></div></xsl:if>
		          		The base path exported by the module's public interface. For example, if you enter /mymodule/
		          		and deploy this module to an HTTP fulcrum module, all requests starting with http://servername/mymodule/
		          		to be routed to this module. 
		        </td>
		      </tr>
			<tr>
			<xsl:if test="$validation/v/internal"><xsl:attribute name="class">error</xsl:attribute></xsl:if>
	        <td class="label">Internal&#160;Path:</td>
	        <td>
	          <input type="text" name="internal" size="50" value="{internal}"/>
	        </td>
	        <td class="comment">
	        	<xsl:if test="$validation/v/internal"><div class="etext"><xsl:value-of select="$validation/v/internal"/></div></xsl:if>
	          Sub-directory in the module source used for resources.
	        </td>
	      </tr>
	      <tr>
	        <td colspan="3">
	          <button class="btn btn-default" type="button" value="&lt; back" onclick="history.go(-1)"><span class="fa fa-arrow-left"> </span>back</button>
	          <xsl:text> </xsl:text>
	          <button class="btn btn-primary" type="submit" value="next &gt;">next <span class="fa fa-arrow-right"> </span></button>
	        </td>
	      </tr>		      
		      
		 </table>
		
	</xsl:template>
	
	
	<xsl:template name="page3">
		<xrl:include>
		 <xrl:identifier>active:controlPanelInfoBoxOpen</xrl:identifier>
		 <xrl:argument name="content">
		   <div>
		   <p>Now we need to select which features your want to include into the new module as starting point.
		   These can always be added later but adding them in now will make life easier.
		   </p>
		   </div>
		 </xrl:argument>
		</xrl:include>
<style>
.ncodeselect {display: none;}
</style>
		<table class="table">
		      <tr>
		        <td class="label">Documentation Stub:</td>
		        <td>
		          <input type="checkbox" name="doc">
		          	<xsl:if test="doc='on'"><xsl:attribute name="checked"/></xsl:if>
		          </input>
		        </td>
		        <td class="comment">Create a the necessary configuration and resources for a documentation stub for the example service.
		        </td>
			</tr>
		      <tr>
		        <td class="label">Unit Test:</td>
		        <td>
		          <input type="checkbox" name="test">
		          	<xsl:if test="test='on'"><xsl:attribute name="checked"/></xsl:if>
		          </input>
		        </td>
		        <td class="comment">This option will create an additional unit test space within the module containing an XUnit test stub
		        for the example service.
		        </td>
			</tr>
			  <tr>
		        <td class="label">Example Script:</td>
		        <td>
		          <select id="selectLang" name="example" size="1">
		          	<option value="noscript">--No Script--</option>
		          	<option value="ncode">
		          		<xsl:if test="example='ncode'"><xsl:attribute name="selected"/></xsl:if>
		          		nCoDE</option>
		          	<option value="groovy">
		          		<xsl:if test="example='groovy'"><xsl:attribute name="selected"/></xsl:if>
		          		Groovy</option>
		          	<option value="dpml">
		          		<xsl:if test="example='DPML'"><xsl:attribute name="selected"/></xsl:if>
		          		DPML</option>
		          </select>
		        </td>
		        <td class="comment">Please choose an implementation language for the example service.
		        </td>
		      </tr>
		      <tr>
		      	<td>
		      		<div class="ncodeselect label">
		      		nCoDE Dynamic Palettes:
		      		</div>
		      	</td>
		      	<td>
		      		<div class="ncodeselect">
		      		<input type="checkbox" name="ncodedi">
		      			<xsl:choose>
		      				<xsl:when test="ncodedi='off'"><xsl:attribute name="checked">false</xsl:attribute></xsl:when>
		      				<xsl:otherwise>
		      					<xsl:attribute name="checked">true</xsl:attribute>
		      				</xsl:otherwise>
		      			</xsl:choose>
		          	</input>
		          	</div>
		      	</td>
		      	<td>
		      	<div class="ncodeselect">
		      	Setup a Dynamic Import hook to automatically allow nCoDE palettes to be discovered and imported.
		      	</div>
		      	</td>
		      </tr>
		      <tr>
		        <td colspan="3">
		          <button class="btn btn-default" type="button" value="&lt; back" onclick="history.go(-1)"><span class="fa fa-arrow-left"> </span>back</button>
		          <xsl:text> </xsl:text>
		          <input class="btn btn-primary" type="submit" value="create"/>
		        </td>
		      </tr>
<script type="text/javascript">
<xsl:comment>
$("#selectLang").change(function(evt)
	{	var v=$(this).attr("value");
		if(v=="ncode")
		{	$(".ncodeselect").slideDown(1000);
		}
		else
		{	$(".ncodeselect").slideUp(1000);
		}
	}
)
</xsl:comment>
</script>		      
		</table>
	</xsl:template>
	
	<xsl:template name="page4">
		<xrl:include>
		 <xrl:identifier>active:controlPanelInfoBoxOpen</xrl:identifier>
		 <xrl:argument name="content">
		   <div>
		   <p>Your module has been successfully created and in a few moments will be commissioned...</p> 
		   <p>It is then ready to try out and extend with your own functionality.</p>
		   <p>The module is located here: <b><xsl:value-of select="source"/></b></p>
		   </div>
		 </xrl:argument>
		</xrl:include>
		<div class="panel panel-default">
			<div class="panel-heading">
				<div class="panel-title">Next Steps</div>
			</div>
			<xsl:if test="fulcrum='HTTPFulcrum' and not(example='noscript') and not (example ='ncode')">
	   		<p><a href="http://localhost:8080{external}service/My Message">Request</a> the example service on the FrontEnd Fulcrum.</p>
	   	</xsl:if>
	   	<xsl:if test="fulcrum='HTTPFulcrum' and not(example='noscript') and (example='ncode')">
	   		<p><a href="http://localhost:8080{external}service">Request</a> the example service on the FrontEnd Fulcrum.</p>
	   	</xsl:if>
	   <xsl:if test="fulcrum='HTTPAdminFulcrum' and not(example='noscript') and not(example='ncode')">
	   		<p><a href="http://localhost:1060{external}service/My Message">Request</a> the example service on the BackEnd Fulcrum.</p>
	   	</xsl:if>
	   	<xsl:if test="fulcrum='HTTPAdminFulcrum' and not(example='noscript') and (example='ncode')">
	   		<p><a href="http://localhost:1060{external}service">Request</a> the example service on the BackEnd Fulcrum.</p>
	   	</xsl:if>
	   <p>View your module in the <a href="/tools/exploreRedirect/{urn}/{version}">Space Explorer</a> to understand the module structure that has been created.</p>
	   <xsl:if test="doc='on'">
	   		<p>View the <a href="/book/view/book:{urn}/">documentation</a> for your module.</p>
	   	</xsl:if>
	   <xsl:if test="test='on'">
	   		<p>View and execute the <a href="/test/view/html/test:{urn}">Unit Tests</a> for your module.</p>
	   	</xsl:if>
		</div>
	   
	   
	</xsl:template>
	
	
	
</xsl:stylesheet> 
