package tools.exceptionHandler;

import javax.servlet.http.HttpServletRequest;

import org.netkernel.http.util.AcceptHeaderParser;
import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.nkf.INKFResponseReadOnly;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.xml.xda.IXDAReadOnly;

public class BEFExceptionHandler extends StandardAccessorImpl
{
	public BEFExceptionHandler()
	{	this.declareThreadSafe();
	}
	
	public void onSource(INKFRequestContext aContext) throws Exception
	{
		//determine if client will desire a HTML formatted error page
		HttpServletRequest httpServletRequest=(HttpServletRequest)aContext.source("httpRequest:/advanced/HttpServletRequest");
		IHDSNode accept=AcceptHeaderParser.getAcceptHeaderState(httpServletRequest);
		boolean desireHTML=false;
		for (Object value : accept.getValues("/headers/Accept/entry/value"))
		{	if (((String)value).contains("html"))
			{	desireHTML=true;
			}
		}
		if (desireHTML)
		{	boolean bef=aContext.getThisRequest().argumentExists("bef");
			if (bef)
			{	onBEF(aContext);
			}
			else
			{	onFEF(aContext);
			}
		}
		else
		{	onRaw(aContext);
		}
	}
	
	public void onRaw(INKFRequestContext aContext) throws Exception
	{
		IXDAReadOnly ex=aContext.source("arg:operand",IXDAReadOnly.class);
		String id="Unknown Error";
		String msg="";
		try
		{	id=ex.getText("(descendant-or-self::ex[id])[last()]/id",true);
			msg=ex.getText("(descendant-or-self::ex[id])[last()]/message",true);
		}
		catch (Exception e) {;}
		StringBuilder sb=new StringBuilder(1024);
		sb.append("Unhandled exception occurred: ");
		sb.append(id); sb.append("\n");
		sb.append(msg); sb.append("\n");
		INKFResponse resp=aContext.createResponseFrom(sb.toString());
	}
	
		
	public void onBEF(INKFRequestContext aContext) throws Exception
	{	
		IXDAReadOnly ex=aContext.source("arg:operand",IXDAReadOnly.class);
		
		INKFRequest req=aContext.createRequest("active:xslt");
		req.addArgument("operator","res:/nkse/style/xsl/format_exception.xsl");
		req.addArgumentByValue("operand",ex);
		INKFResponseReadOnly respIn=aContext.issueRequestForResponse(req);
		INKFResponse resp=aContext.createResponseFrom(respIn);

		String id="Unknown Error";
		String msg="";
		try
		{	id=ex.getText("(descendant-or-self::ex[id])[last()]/id",true);
			msg=ex.getText("(descendant-or-self::ex[id])[last()]/message",true);
		}
		catch (Exception e) {;}
		
		HDSBuilder b=new HDSBuilder();
		b.pushNode("view");
		b.addNode("title",id);
		b.addNode("subtitle", msg);
		b.pushNode("icon", "fa fa-exclamation-triangle"); b.addNode("@type", "fa"); b.popNode();
		
		resp.setHeader("WrappedControlPanel", true);
		resp.setHeader("WrappedControlHeading", b.getRoot());
	}
	
	public void onFEF(INKFRequestContext aContext) throws Exception
	{	
		IXDAReadOnly ex=aContext.source("arg:operand",IXDAReadOnly.class);
		
		INKFRequest req=aContext.createRequest("active:xslt");
		req.addArgument("operator","res:/nkse/style/xsl/format_exception_full.xsl");
		req.addArgumentByValue("operand",ex);
		INKFResponseReadOnly respIn=aContext.issueRequestForResponse(req);
		INKFResponse resp=aContext.createResponseFrom(respIn);
	}

}
