<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xrl="http://netkernel.org/xrl">
	<xsl:output method="xml"/>
	
	<xsl:template match="/k">
		<div>
		<xrl:include>
			<xrl:identifier>active:controlPanelInfoBox</xrl:identifier>
			<xrl:argument name="content">
				<div>
				<p>
          The Requests and Threads tool provides a view of the internal state of the NetKernel microkernel.
        </p>
				<p>
          It shows the currently active root requests and how they translate onto the physical execution threads.
        </p>
				<p>
          The <b>Root Requests</b> section lists all active transport initiated requests. The status
				column shows a diagnostic execution status which may be one of  OK, QUEUED, BUSY_ENDPOINT,
				RUNAWAY, DEADLOCKED, RESPONSELESS, LONG_WAIT, LONG_IO_WAIT or THREAD_STARVED.
          Clicking on the button labeled &quot;kill&quot; next to a root request will
          cause NetKernel to make its best attempt to terminate execution of that request.
          (Timings are expressed in milliseconds)
        </p>
				<p>
          The <b>Kernel Worker Threads</b> section shows all kernel and transport threads
          available to service requests.
          Clicking on a thread to see a detailed stack trace and details of any held locks.
          Threads may be in the states of  RUNNABLE, BLOCKED, WAITING, TIMED_WAITING, NEW or TERMINATED.
          See
				  <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Thread.State.html">Thread.State</a>
          for more information.
        </p>
				<p>
          The <b>Locked Endpoints</b> section shows any non-threadsafe endpoints which are handling requests
          along with requests they are blocking.
        </p>
				<p>
          The <b>Request Queue</b> section shows requests waiting a worker thread.
          This queue is typically empty unless a system uses a large number of asynchronous requests.
        </p>
				<p>
         The <b>Other JVM Threads</b> section lists all the other threads within the Java virtual machine
          along with their status.
        </p>
				</div>
			</xrl:argument>
		</xrl:include>
		
		<div id="linkrequests" class="panel panel-default">
				<div class="panel-heading">
					<div class="panel-title">Root Request</div>
				</div>
		<table class="table-striped table-condensed">
		<tr>
			<th>Root Id</th><th>Request</th><th>Duration</th><th>Last Activity</th><th>Status</th><th/>
		</tr>
		<xsl:for-each select="roots/state">
			<xsl:sort select="root-id" data-type="number"/>
			<tr>	
				<xsl:if test="status!='OK'">
					<xsl:attribute name="class">error</xsl:attribute>
				</xsl:if>
				<td><xsl:value-of select="root-id"/></td>
				<td><xsl:value-of select="request"/>
				</td>
				<td><xsl:value-of select="duration"/></td>
				<td><xsl:value-of select="active"/></td>
				<td><xsl:value-of select="statusString"/></td>
				<td>
					<input type="button" value="kill" onclick="parent.location='/tools/kill/{root-id}'" class="btn btn-sm btn-danger"/>
				</td>
			</tr>
		</xsl:for-each>
		</table>
		</div>		
		
		<div id="linkworkerthreads" class="panel panel-default">
				<div class="panel-heading">
					<div class="panel-title">Kernel Worker Threads</div>
				</div>
		<table class="table-striped table-condensed">
		<tr>
			<th>Thread</th><th>Status</th><th>CPU</th><th>Root</th><th>Last Activity</th>
		</tr>
		<xsl:for-each select="threads/thread[isKernel='true']">
			<xsl:sort select="name"/>
			<tr>	
				<td><a href="#{name}"><xsl:value-of select="name"/></a></td>
				<td><xsl:value-of select="state"/></td>
				<td><xsl:value-of select="cpu"/></td>
				<td><xsl:value-of select="root"/></td>
				<td><xsl:value-of select="request"/>
				</td>
			</tr>
		</xsl:for-each>
		</table>
		</div>

		<div id="linklockedep" class="panel panel-default">
				<div class="panel-heading">
					<div class="panel-title">Locked Endpoints</div>
				</div>
		<table class="table-striped table-condensed">
		<tr>
			<th>Endpoint/Request</th><th>Root</th><th>Queued Duration</th>
		</tr>
		<xsl:choose>
			<xsl:when test="locked-endpoints/endpoint">
				<xsl:for-each select="locked-endpoints/endpoint">
					<tr>	
						<td><xsl:value-of select="class"/></td>
					</tr>
					<xsl:for-each select="queued/state">
						<tr>
							<td style="padding-left: 1em;"><xsl:value-of select="request"/></td>
							<td><xsl:value-of select="root-id"/></td>
							<td><xsl:value-of select="active"/></td>
						</tr>
					</xsl:for-each>
				</xsl:for-each>
			</xsl:when>
			<xsl:otherwise>
				<tr>	
					<td colspan="3">- No results -</td>
				</tr>
			</xsl:otherwise>
		</xsl:choose>
		</table>
		</div>

		<div id="linkrequestqueue" class="panel panel-default">
				<div class="panel-heading">
					<div class="panel-title">Request Queue</div>
				</div>
		<table class="table-striped table-condensed">
		<tr>
			<th>Priority</th><th>Request</th><th>Root</th><th>State</th><th>Queued Duration</th>
		</tr>
		<xsl:choose>
			<xsl:when test="queue/state">
				<xsl:for-each select="queue/state">
					<tr>	
						<td><xsl:value-of select="priority"/></td>
						<td><xsl:value-of select="request"/></td>
						<td><xsl:value-of select="root-id"/></td>
						<td><xsl:value-of select="state"/></td>
						<td><xsl:value-of select="active"/></td>
					</tr>
				</xsl:for-each>
			</xsl:when>
			<xsl:otherwise>
				<tr>	
					<td colspan="3">- No results -</td>
				</tr>
			</xsl:otherwise>
		</xsl:choose>
		</table>
		</div>
		
		<div id="linkjvmthreads" class="panel panel-default">
				<div class="panel-heading">
					<div class="panel-title">JVM Threads</div>
				</div>

		<table class="table-striped table-condensed">
		<tr>
			<th>Thread</th><th>Status</th><th>CPU</th>
		</tr>
		<xsl:for-each select="threads/thread[isKernel!='true']">
			<xsl:sort select="name"/>
			<tr>	
				<td><a href="#{name}"><xsl:value-of select="name"/></a></td>
				<td><xsl:value-of select="state"/></td>
				<td><xsl:value-of select="cpu"/></td>
			</tr>
		</xsl:for-each>
		</table>
		</div>

		<div id="linkjvmthreads" class="panel panel-default">
				<div class="panel-heading">
					<div class="panel-title">Thread Detail</div>
				</div>
		<table class="table-striped table-condensed">
		<tr>
			<th>Thread</th><th>Status</th><th>Root</th><th>Last Activity</th>
		</tr>
		<xsl:for-each select="threads/thread">
			<xsl:sort select="name"/>
			<tr>	
				<td id="{name}"><xsl:value-of select="name"/></td>
				<td><xsl:value-of select="state"/></td>
				<td><xsl:value-of select="root"/></td>
				<td><xsl:value-of select="request"/></td>
			</tr>
			<xsl:if test="lock">
				<tr><td/>
					<td colspan="4">Lock name: [<xsl:value-of select="lock"/>] Lock owner: [<xsl:value-of select="lock-owner"/>]
					</td>
				</tr>
			</xsl:if>
			
			<tr>
				<td/>
				<td colspan="4">
					<div style="font-size: 66%">
					<xsl:for-each select="stack/frame">
						<div><xsl:value-of select="."/></div>
					</xsl:for-each>
					</div>
				</td></tr>
		</xsl:for-each>
		</table>
		</div>
		</div>
	</xsl:template>
	
</xsl:stylesheet>
