package tools.mupload;

import java.net.URI;

import org.netkernel.container.config.IConfiguration;
import org.netkernel.ext.system.representation.IRepDeployedModules;
import org.netkernel.layer0.boot.BootUtils;
import org.netkernel.layer0.boot.IModule;
import org.netkernel.layer0.boot.ModuleManager;
import org.netkernel.layer0.boot.ModuleManager.MOD_DECL_TYPE;
import org.netkernel.layer0.boot.ModuleSourceWithLevel;
import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.nkf.INKFResponseReadOnly;
import org.netkernel.mod.hds.HDSFactory;
import org.netkernel.mod.hds.IHDSDocument;
import org.netkernel.mod.hds.IHDSMutator;
import org.netkernel.mod.hds.IHDSReader;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.xml.xda.IXDA;
import org.netkernel.xml.xda.IXDAIterator;
import org.netkernel.xml.xda.IXDAReadOnly;

public class ModuleDeleteAccessor extends StandardAccessorImpl
{
	public ModuleDeleteAccessor()
	{	this.declareThreadSafe();
	}
	
	public void onSource(INKFRequestContext aContext) throws Exception
	{
		String moduleHash=aContext.getThisRequest().getArgumentValue("entityId");
		String action=aContext.source("httpRequest:/param/action",String.class);
		if (action==null)
		{	onStatus(moduleHash,aContext);
		}
		else if (action.equals("remove"))
		{	onRemove(moduleHash,aContext);
		}
	}
	
	public void onStatus(String aHash, INKFRequestContext aContext) throws Exception
	{
		//determine type of module
		IRepDeployedModules modules=aContext.source("active:moduleList",IRepDeployedModules.class);
		IModule module=(IModule)modules.componentForHash(aHash);
		URI moduleSource=module.getSource();
		ModuleManager mm=ModuleManager.getSingleton();
		ModuleSourceWithLevel mswl=mm.getRegisteredModules().get(moduleSource);
		String moduleDeclaration=mswl.getDeclaration();
		MOD_DECL_TYPE moduleDeclType=mm.getDeclarationTypeFor(moduleDeclaration);
		
		//format page of options
		IHDSMutator m=HDSFactory.newDocument()
				.pushNode("delete")
				.addNode("name", module.getMeta().getName())
				.addNode("moduleDeclaration", moduleDeclaration)
				.addNode("declarationType", moduleDeclType.toString());
		INKFRequest req=aContext.createRequest("active:xslt");
		req.addArgument("operator", "res:/tools/mupload/styleDeleteStatus.xsl");
		req.addArgumentByValue("operand", m.toDocument(false));
		req.setRepresentationClass(aContext.getThisRequest().getRepresentationClass());
		INKFResponseReadOnly resp=aContext.issueRequestForResponse(req);
		INKFResponse resp2=aContext.createResponseFrom( resp );
	    resp2.setHeader("WrappedControlPanel", true);
	}
	
	public void onRemove(String aHash, INKFRequestContext aContext) throws Exception
	{
		//determine module declaration file
		IRepDeployedModules modules=aContext.source("active:moduleList",IRepDeployedModules.class);
		IModule module=(IModule)modules.componentForHash(aHash);
		URI moduleSource=module.getSource();
		ModuleManager mm=ModuleManager.getSingleton();
		ModuleSourceWithLevel mswl=mm.getRegisteredModules().get(moduleSource);
		String moduleDeclaration=mswl.getDeclaration();
		
		//load modules file
		IXDA modulesXML = aContext.source(moduleDeclaration,IXDAReadOnly.class).getClone();
		
		//remove module
		IConfiguration config=aContext.getKernelContext().getKernel().getConfiguration();
		URI base = URI.create(BootUtils.getInstallPath(config));
		String moduleSourceString=moduleSource.toString();
		IXDAIterator i=modulesXML.iterator("/modules/module");
		while(i.next())
		{	String p=i.getText(".", true);
			URI a=base.resolve(p);
			if (a.toString().equals(moduleSourceString))
			{	i.rename(".", "disabled");
			}
		}
		
		//style final page
		IHDSMutator m=HDSFactory.newDocument();
		m.pushNode("status").addNode("source", mswl.getSource().toString());
		INKFRequest req=aContext.createRequest("active:xslt");
		req.addArgument("operator", "res:/tools/mupload/styleDeleteFinal.xsl");
		req.addArgumentByValue("operand", m.toDocument(false));
		req.setRepresentationClass(aContext.getThisRequest().getRepresentationClass());
		INKFResponseReadOnly resp=aContext.issueRequestForResponse(req);
		INKFResponse resp2=aContext.createResponseFrom( resp );
	    resp2.setHeader("WrappedControlPanel", true);		
		
		//async save declaration file
		req=aContext.createRequest("active:delayedSaveModuleDecl");
		req.addArgumentByValue("declarationURI", moduleDeclaration);
		req.addArgumentByValue("declaration", modulesXML);
		aContext.issueAsyncRequest(req);
		
		
		
		
	}

}
