package tools.mupload;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URI;
import java.nio.file.attribute.AclEntryPermission;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import java.util.jar.JarInputStream;

import org.netkernel.container.IKernel;
import org.netkernel.container.config.IConfiguration;
import org.netkernel.ext.system.representation.IRepDeployedModules;
import org.netkernel.layer0.boot.BootUtils;
import org.netkernel.layer0.boot.IModule;
import org.netkernel.layer0.boot.ModuleManager;
import org.netkernel.layer0.boot.ModuleManager.MOD_DECL_TYPE;
import org.netkernel.layer0.boot.ModuleSourceWithLevel;
import org.netkernel.layer0.nkf.INKFAsyncRequestListener;
import org.netkernel.layer0.nkf.INKFLocale;
import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.nkf.INKFResponseReadOnly;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.representation.IReadableBinaryStreamRepresentation;
import org.netkernel.layer0.util.Utils;
import org.netkernel.layer0.util.XMLReadable;
import org.netkernel.layer0.util.XMLUtils;
import org.netkernel.mod.hds.HDSFactory;
import org.netkernel.mod.hds.IHDSDocument;
import org.netkernel.mod.hds.IHDSMutator;
import org.netkernel.mod.hds.IHDSReader;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.urii.impl.Version;
import org.netkernel.xml.xda.DOMXDA;
import org.netkernel.xml.xda.IXDA;
import org.netkernel.xml.xda.IXDAIterator;
import org.netkernel.xml.xda.IXDAReadOnly;

public class ModuleUploadAccessor extends StandardAccessorImpl
{
	public ModuleUploadAccessor()
	{	this.declareThreadSafe();
	}
	
	public void onSource(INKFRequestContext aContext) throws Exception
	{
		IHDSReader params = aContext.source("httpRequest:/params",IHDSDocument.class).getReader();
		String uploaded=(String)params.getFirstValueOrNull("/uploaded");
		String action=(String)params.getFirstValueOrNull("/action");
		if (uploaded!=null)
		{	onUpload(uploaded,aContext);
		}
		else if (action!=null)
		{	
			if ("check".equals(action))
			{	onCheck(params,aContext);
			}
			else
			{	onAction(action,params,aContext);
			}
		}	
		else
		{	onIndex(aContext);
		}
	}
	
	

	public void onIndex(INKFRequestContext aContext) throws Exception
	{
		INKFResponseReadOnly page=aContext.sourceForResponse("res:/tools/mupload/index.xml");
		INKFResponse resp=aContext.createResponseFrom(page);
		resp.setHeader("WrappedControlPanel", true);
	}
	public void onUpload(String aUpload, INKFRequestContext aContext) throws Exception
	{
		String unjar=aContext.source("httpRequest:/param/unjar",String.class);		
		String uploadURI="httpRequest:/upload/uploaded";
		//System.out.println(uploadURI);
		IReadableBinaryStreamRepresentation data=aContext.source(uploadURI,IReadableBinaryStreamRepresentation.class);
		//System.out.println("got "+aUpload+" with "+data.getContentLength()+" bytes");

		
		File f=File.createTempFile("ModuleUpload", ".jar");
		f.deleteOnExit();
		FileOutputStream fos=new FileOutputStream(f);
		data.write(fos);
		fos.flush();
		fos.close();
		
		
		IHDSMutator m=HDSFactory.newDocument();
		try
		{
		
			//get module details
			URI moduleURI=f.toURI();
			XMLReadable defn=BootUtils.getXMLFrom(moduleURI, "module.xml");
			String uri = defn.getText("/module/meta/identity/uri");
			String version = defn.getText("/module/meta/identity/version");
			String name = defn.getText("/module/meta/info/name");
			String desc = defn.getText("/module/meta/info/description");
			
		
			//find existing modules
			IKernel k=aContext.getKernelContext().getKernel();
			List<IModule> modules=new ArrayList<IModule>();
			ModuleManager mm=ModuleManager.getSingleton();
			Version maxVersion=null;
			for (IModule module : mm.getModules())
			{
				String moduleId=module.getMeta().getIdentifier();
				if (moduleId.equals(uri))
				{	modules.add(module);
					Version v=new Version(module.getMeta().getVersion());
					if (maxVersion==null || v.isGreaterThan(maxVersion))
					{	maxVersion=v;
					}
				}
			}
			int versionCompare=0;
			if (maxVersion!=null)
			{	versionCompare=new Version(version).compareTo(maxVersion);
			}
			
			//find where declaration of existing modules is - module.xml or modules.d
			MOD_DECL_TYPE type=null;
			boolean mixedTypes=false;
			String declarationURI=null;
			for (IModule module: modules)
			{
				declarationURI=mm.getRegisteredModules().get(module.getSource()).getDeclaration();
				MOD_DECL_TYPE declType=mm.getDeclarationTypeFor(declarationURI);
				if (type==null || type==declType)
				{	type=declType;
				}
				else
				{	mixedTypes=true;
				}
			}
			
			// find existing modules.d
			IConfiguration config=aContext.getKernelContext().getKernel().getConfiguration();
			URI base = URI.create(BootUtils.getInstallPath(config));
			boolean createModulesd=false;
			if (type==null)
			{
				URI modulesdir = BootUtils.getKernelURIProperty(config,"netkernel.init.modulesdir");
				if (modulesdir == null)
				{	modulesdir = URI.create("etc/modules.d/"); // Default to the generic named directory
				}
				if (!modulesdir.isAbsolute())
				{	modulesdir = base.resolve(modulesdir);
				}
				File modulesdirfile = new File(modulesdir);
				if (!modulesdirfile.exists())
				{	modulesdirfile.mkdirs();
				}
				File[] declFiles=modulesdirfile.listFiles(new FilenameFilter()
				{	public boolean accept(File dir, String name)
					{	return name.endsWith(".xml");
					}
				});
				if (declFiles.length>0)
				{	long ts=0;
					File newest=null;
					for (File f2: declFiles)
					{	if (f2.lastModified()>ts)
						{	newest=f2;
							ts=f2.lastModified();
						}
					}
					declarationURI=newest.toURI().toString();
					type=MOD_DECL_TYPE.MODULESD;
				}
				else
				{	//need to create
					File f2=new File(modulesdirfile,"modules.xml");
					declarationURI=f2.toURI().toString();
					type=MOD_DECL_TYPE.MODULESD;
					createModulesd=true;
				}
			}
			
			//module directory
			String moduleDir=base.resolve("modules/").toString();
			
			//list of existing module sources to replace
			StringBuilder em=new StringBuilder();
			for (IModule module: modules)
			{
				em.append(module.getSource().toString());
				em.append(' ');
			}
			
			
			m.pushNode("status")
				.pushNode("upload")
					.addNode("uri",uri)
					.addNode("version",version)
					.addNode("name",name)
					.addNode("desc",desc)
					.addNode("filename",aUpload)
					.addNode("tmpFile", f.toURI().toString())
					.addNode("unjar", unjar==null?"false":"true")
					.addNode("timestamp", Long.toString(System.currentTimeMillis()))
				.popNode()
				.pushNode("existing")
					.addNode("count",modules.size())
					.addNode("maxVersion",maxVersion!=null?maxVersion.toString():null)
					.addNode("versionCompare",versionCompare)
					.addNode("declarationType",type!=null?type.toString():null)
					.addNode("declarationURI",declarationURI)
					.addNode("mixedLocations",mixedTypes)
					.addNode("moduleDir",moduleDir)
					.addNode("createModulesd",createModulesd)
					.addNode("replaceSources",em.toString())
				.popNode()
			.popNode();
			
		}
		catch (Exception e)
		{
			m.pushNode("error");
			if (e.getClass()==FileNotFoundException.class)
			{	m.addNode("msg", "Upload doesn't look like a NetKernel module - no module.xml");
			}
			else if (e.getClass()==java.util.zip.ZipException.class)
			{	m.addNode("msg", "Upload doesn't look like a NetKernel module - not a JAR File");
			}
			else
			{	m.addNode("msg", e.getClass()+" "+e.getMessage());
				aContext.logRaw(INKFLocale.LEVEL_WARNING, Utils.throwableToString(e));
			}
		}

			
		IHDSDocument operand=m.toDocument(false);
		//System.out.println(operand);
		
		INKFRequest req=aContext.createRequest("active:xslt");
		req.addArgument("operator", "res:/tools/mupload/styleStatus.xsl");
		req.addArgumentByValue("operand", operand);
		req.setRepresentationClass(aContext.getThisRequest().getRepresentationClass());
		INKFResponseReadOnly resp=aContext.issueRequestForResponse(req);
		INKFResponse resp2=aContext.createResponseFrom( resp );
	    resp2.setHeader("WrappedControlPanel", true);
	}
	
	public void onAction(String aAction, IHDSReader aParams, INKFRequestContext aContext) throws Exception
	{
		String action=(String)aParams.getFirstValueOrNull("/action");
		String tmpFile=(String)aParams.getFirstValueOrNull("/tmpFile");
		String declarationURI=(String)aParams.getFirstValueOrNull("/declarationURI");
		String filename=(String)aParams.getFirstValueOrNull("/filename");
		String moduleDir=(String)aParams.getFirstValueOrNull("/moduleDir");
		String replaceSources=(String)aParams.getFirstValueOrNull("/replaceSources");
		String createModulesd=(String)aParams.getFirstValueOrNull("/createModulesd");
		String uri=(String)aParams.getFirstValueOrNull("/uri");
		String version=(String)aParams.getFirstValueOrNull("/version");
		boolean unjar="true".equals(aParams.getFirstValueOrNull("/unjar"));
		long timestamp=Long.parseLong((String)aParams.getFirstValue("/timestamp"));
		File moduleFile=null;
		
		if (System.currentTimeMillis()-timestamp>30000)
		{
			aContext.logRaw(INKFLocale.LEVEL_WARNING, "Module upload timed out, please try again");
		}
		else
		{
		
		
			//System.out.println(aAction+" "+tmpFile+" "+declarationURI+" "+filename);
			
			File moduleDirFile=new File(URI.create(moduleDir));
			if (unjar)
			{	int i=filename.lastIndexOf('.');
				if (i>0)
				{	filename=filename.substring(0,i);
				}
			}
	
			moduleFile=new File(moduleDirFile,filename);
			int n=1;
			while(moduleFile.exists())
			{	int i=filename.lastIndexOf('.');
				String uniqueFilename=filename.substring(0, i)+"."+Integer.toString(n++)+filename.substring(i);
				moduleFile=new File(moduleDirFile,uniqueFilename);
			}
			
			if (unjar)
			{	JarFile jar = new JarFile(new File(URI.create(tmpFile)));
				Enumeration enumEntries = jar.entries();
				while (enumEntries.hasMoreElements())
				{
				    JarEntry file = (JarEntry) enumEntries.nextElement();
				    File f = new File(moduleFile, file.getName());
				    if (file.isDirectory())
				    { // if its a directory, create it
				        //f.mkdir();
				        continue;
				    }
				    else
				    {	f.getParentFile().mkdirs();
				    }
				    InputStream is = jar.getInputStream(file); // get the input stream
				    OutputStream fos = new BufferedOutputStream(new FileOutputStream(f));
				    while (is.available() > 0)
				    {  // write contents of 'is' to 'fos'
				        fos.write(is.read());
				    }
				    fos.close();
				    is.close();
				}			
			}
			else
			{	//copy file
				FileOutputStream fos=new FileOutputStream(moduleFile);
				FileInputStream fis=new FileInputStream(new File(URI.create(tmpFile)));
				Utils.pipe(fis, fos);
			}
			
			//determine relative URI for module source
			IConfiguration config=aContext.getKernelContext().getKernel().getConfiguration();
			URI base = URI.create(BootUtils.getInstallPath(config));
			String relativePath=base.relativize(moduleFile.toURI()).toString();
			
			IXDA modulesXML;
			if ("true".equals(createModulesd))
			{	//create new modules.d file
				modulesXML= new DOMXDA(XMLUtils.newDocument());
				modulesXML.appendPath("/", "modules", null);
			}
			else
			{	//load up declaration file
				modulesXML = aContext.source(declarationURI,IXDAReadOnly.class).getClone();
			}
			
			
			//disable any module declarations that we replace
			if ("replace".equals(action))
			{	String[] replaceArray=Utils.splitString(replaceSources, " ");
				for (String replace : replaceArray)
				{	IXDAIterator i=modulesXML.iterator("/modules/module");
					while(i.next())
					{	String p=i.getText(".", true);
						URI a=base.resolve(p);
						if (a.toString().equals(replace))
						{
							i.rename(".", "disabled");
						}
					}
				}
			}
			
			//add new declaration at end
			String relativeModuleFile=base.relativize(moduleFile.toURI()).toString();
			modulesXML.appendPath("/modules", "module", relativeModuleFile);
			
			//serialize declaration file
			INKFRequest req=aContext.createRequest("active:serializeDOM");
			req.addArgumentByValue("operand", modulesXML);
			req.addArgumentByValue("operator", "<serialize><indent/></serialize>");
			Object rep=aContext.issueRequest(req);
			
			
			
			//async save declaration file
			req=aContext.createRequest("active:delayedSaveModuleDecl");
			req.addArgumentByValue("declarationURI", declarationURI);
			req.addArgumentByValue("declaration", rep);
			aContext.issueAsyncRequest(req).setListener(new INKFAsyncRequestListener()
			{	public void receiveResponse(INKFResponseReadOnly aResponse, INKFRequestContext aContext) throws Exception
				{	aContext.setNoResponse();
				}
				public void receiveException(NKFException aException, INKFRequest aRequest, INKFRequestContext aContext) throws Exception
				{	aContext.logRaw(INKFLocale.LEVEL_WARNING, Utils.throwableToString(aException));
					aContext.setNoResponse();
				}
			});
			
		}
		
		//generate final page
		IHDSMutator m=HDSFactory.newDocument();
		m.pushNode("status");
		if (moduleFile!=null) m.addNode("source", moduleFile.toURI().toString());
		
		INKFRequest req=aContext.createRequest("active:xslt");
		req.addArgument("operator", "res:/tools/mupload/styleFinal.xsl");
		req.addArgumentByValue("operand", m.toDocument(false));
		req.setRepresentationClass(aContext.getThisRequest().getRepresentationClass());
		INKFResponseReadOnly resp=aContext.issueRequestForResponse(req);
		INKFResponse resp2=aContext.createResponseFrom( resp );
	    resp2.setHeader("WrappedControlPanel", true);
		
		
		
	}
	
	public void onCheck(IHDSReader aParams, INKFRequestContext aContext) throws Exception
	{
		String source=(String)aParams.getFirstValueOrNull("/source");
		//System.out.println("onCheck "+uri+" "+version+" "+source);
		
		URI sourceURI=URI.create(source);
		ModuleManager mm=ModuleManager.getSingleton();
		
		long syncTime=mm.getLastSyncTime();
		long now=System.currentTimeMillis();
		long secSinceSync=(now-syncTime)/1000L;
		
		int status=0;
		String hash=null;
		if (secSinceSync>10)
		{	status=0;
		}
		else
		{	ModuleSourceWithLevel mswl=mm.getRegisteredModules().get(sourceURI);
			if (mswl==null)
			{	status=1;
			}
			else
			{	
				IModule found=null;
				for (IModule mod : mm.getModules())
				{	if (mod.getSource().equals(sourceURI))
					{	found=mod;
						break;
					}
				}
				if (found!=null)
				{	IRepDeployedModules modules=aContext.source("active:moduleList", IRepDeployedModules.class);
					hash=modules.hashForComponent(found);
				}
				if (hash==null)
				{	status=2;
				}
				else
				{	status=3;
				}
			}
		}
		StringBuilder result=new StringBuilder();
		result.append("{\"status\":");
		result.append(Integer.toString(status));
		if (hash!=null)
		{	result.append(", \"hash\":\"");
			result.append(hash);
			result.append("\"");
		}
		result.append("}");
		aContext.createResponseFrom(result.toString()).setExpiry(INKFResponse.EXPIRY_ALWAYS);
	}

}
