<?xml version="1.0" encoding="UTF-8" ?>

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xrl="http://netkernel.org/xrl">
	<xsl:output method="xml" />

	<xsl:template match="/error" >
		<div>
			<div class="alert alert-danger">
				<p><b><xsl:value-of select="msg"/></b></p>
				<p>Please correct problem and then try upload again.</p>
			</div>
			<div>
				<a class="btn btn-primary" href="/tools/mupload/">select another file</a>
			</div>
		</div>
	</xsl:template>

	<xsl:template match="/status" >
		<div>
			<div>Uploaded module:</div>
			<div class="alert alert-info">
				<div style="display: table-row;">
					<div style="float: left; display:table-cell; vertical-align:middle;">
						<img width="48" border="0" height="48" style="margin-right: 4px; margin-left: 2px;" src="/tools/introspect/pub/deployment2.png"/>
					</div>
					<div style="display:table-cell; vertical-align:middle;">
						<div class="template-icon-title"><xsl:value-of select="upload/name"/></div>
						<div class="template-icon-subtitle"><xsl:value-of select="upload/desc"/></div>
						<div class="template-icon-subtitle">identifier: <xsl:value-of select="upload/uri"/></div>
						<div class="template-icon-subtitle">version: <xsl:value-of select="upload/version"/></div>
					</div>
				</div>
			</div>
			
			<div>Action to be performed:</div>
			<form method="post">
			<!-- hidden fields for all data needed to complete process -->
			<input type="hidden" name="tmpFile" value="{upload/tmpFile}"/>
			<input type="hidden" name="filename" value="{upload/filename}"/>
			<input type="hidden" name="uri" value="{upload/uri}"/>
			<input type="hidden" name="version" value="{upload/version}"/>
			<input type="hidden" name="unjar" value="{upload/unjar}"/>
			<input type="hidden" name="declarationURI" value="{existing/declarationURI}"/>
			<input type="hidden" name="moduleDir" value="{existing/moduleDir}"/>
			<input type="hidden" name="replaceSources" value="{existing/replaceSources}"/>
			<input type="hidden" name="createModulesd" value="{existing/createModulesd}"/>
			<input type="hidden" name="timestamp" value="{upload/timestamp}"/>
			
			<xsl:choose>
			
				<!-- error conditions -->
				<xsl:when test="existing/versionCompare='1'">
					<xsl:call-template name="error">
						<xsl:with-param name="message">Newer version of this module (<xsl:value-of select="existing/maxVersion"/>) already exists</xsl:with-param>
					</xsl:call-template>
				</xsl:when>
				<xsl:when test="existing/declarationType='STEM'">
					<xsl:call-template name="error">
						<xsl:with-param name="message">This module is declared as a core Stem module and cannot be updated with this tool.</xsl:with-param>
					</xsl:call-template>
				</xsl:when>
				<xsl:when test="existing/mixedLocations='true'">
					<xsl:call-template name="error">
						<xsl:with-param name="message">Versions of this module are declared in multiple locations including Stem (stem-system.conf), Apposite (modules.xml) and modules.d.</xsl:with-param>
					</xsl:call-template>
				</xsl:when>
				
				<xsl:when test="existing/count=0">
					<xsl:call-template name="good">
						<xsl:with-param name="message">This is a new module not currently deployed on this instance</xsl:with-param>
						<xsl:with-param name="buttons">install</xsl:with-param>
					</xsl:call-template>
				</xsl:when>
				<xsl:when test="existing/count>=1 and existing/versionCompare='0'">
					<xsl:call-template name="good">
						<xsl:with-param name="message">Existing module with same version currently deployed</xsl:with-param>
						<xsl:with-param name="buttons">replace</xsl:with-param>
					</xsl:call-template>
				</xsl:when>
				<xsl:when test="existing/count=1 and existing/versionCompare='-1'">
					<xsl:call-template name="good">
						<xsl:with-param name="message">Older version <xsl:value-of select="existing/maxVersion"/> of module is already deployed</xsl:with-param>
						<xsl:with-param name="buttons">install replace</xsl:with-param>
					</xsl:call-template>
				</xsl:when>
				<xsl:when test="existing/count>1 and existing/versionCompare='-1'">
					<xsl:call-template name="good">
						<xsl:with-param name="message">Multiple older versions of module are already deployed</xsl:with-param>
						<xsl:with-param name="buttons">install replace</xsl:with-param>
					</xsl:call-template>
				</xsl:when>
				
				
			</xsl:choose>
			</form>
		</div>
	</xsl:template>
	
	<xsl:template name="error" >
		<xsl:param name="message"/>
		<div>
			<div class="alert alert-danger">
				<p><b><xsl:value-of select="$message"/></b></p>
				<p>Please correct problem and then try upload again.</p>
			</div>
			<div>
				<a class="btn btn-primary" href="/tools/mupload/">cancel</a>
			</div>
		</div>
	</xsl:template>
		
	<xsl:template name="good" >
		<xsl:param name="message"/>
		<xsl:param name="buttons"/>
		<div>
			<div class="alert alert-success">
				<p><b><xsl:value-of select="$message"/></b></p>
				<p>This module is ready to be installed. If you proceed the following actions will be performed:</p>
				<ol>
					<li>The uploaded module <b><xsl:value-of select="upload/filename"/></b>
					will be copied to <b><xsl:value-of select="existing/moduleDir"/></b> directory.</li>
					<xsl:if test="upload/unjar='true'">
						<li>The module jar file will be expanded.</li>
					</xsl:if>
					<xsl:if test="existing/createModulesd='true'">
						<li>Create new modules.d declaration file <b><xsl:value-of select="existing/declarationURI"/></b>.</li>
					</xsl:if>
					<li>The 
						<xsl:choose>
							<xsl:when test="existing/declarationType='APPOSITE'">Apposite module declaration file</xsl:when>
							<xsl:when test="existing/declarationType='MODULESD'">modules.d module declaration file</xsl:when>
						</xsl:choose> 
					module declaration file <b><xsl:value-of select="existing/declarationURI"/></b> will be updated to 
					<xsl:choose>
						<xsl:when test="contains($buttons,'replace') and contains($buttons,'install')">install or replace existing</xsl:when>
						<xsl:when test="contains($buttons,'replace')">replace existing</xsl:when>
						<xsl:when test="contains($buttons,'install')">install</xsl:when>
					</xsl:choose> module.</li>
					<li>NetKernel will hot deploy new module.</li>
				</ol>
			</div>
			
			<xsl:if test="existing/declarationType='APPOSITE'">
				<div class="alert alert-danger">
					<p><b>Warning!</b></p>
					<p>Changes made to Apposite module declaration file
					<b><xsl:value-of select="existing/declarationURI"/></b> may be overwritten when you install or
					update modules using Apposite. Apposite module declaration file should only be used for NetKernel
					system modules.</p>
				</div>
			</xsl:if>
			
			<div>
				<a class="btn btn-default" href="/tools/mupload/">cancel</a>

				<xsl:if test="contains($buttons,'replace')">
					<xsl:text> </xsl:text>
					<input type="submit" class="btn btn-primary" name="action" value="replace"/>
				</xsl:if>
				<xsl:if test="contains($buttons,'install')">
					<xsl:text> </xsl:text>
					<input type="submit" class="btn btn-primary" name="action" value="install"/>
				</xsl:if>
				
			</div>
		</div>
	</xsl:template>

	
</xsl:stylesheet> 
